syntax = "proto3";

import "other.proto";

message FirstOneof {}

message Bar {
  oneof content {
    FirstOneof first = 1;
    SecondOneof second = 2; // unknown size if no options are considered
  }
}

message Foo {
  AnotherList foo = 1;      // again, unknown size
  Bar bar = 2;              // no duplicate size_union shall be generated anymore
}


