/* Test nanopb option parsing.
 * options.expected lists the patterns that are searched for in the output.
 */

syntax = "proto2";

import "nanopb.proto";

message Message1
{
    required bytes data = 1 [(nanopb).type = FT_INLINE, (nanopb).max_size = 32];
}

message Message2
{
    optional bytes data = 1 [(nanopb).type = FT_INLINE, (nanopb).max_size = 64];
}
