syntax = "proto2";
import 'nanopb.proto';

message SubMessage {
    repeated int32 foo = 1 [(nanopb).type = FT_POINTER];
}

message MainMessage {
    repeated SubMessage bar = 1 [(nanopb).max_count = 5];
}

