syntax = "proto2";
import "nanopb.proto";

message Extendable {
    extensions 10 to 100;
}

extend Extendable {
    optional int32 opt_int32 = 11;
}

message PointerMessage {
    optional int32 opt_int32 = 11 [(nanopb).type = FT_POINTER];
}

