syntax = "proto2";

enum SignedEnum {
    SE_MIN = -1;
    SE_MAX = 255;
}

enum UnsignedEnum {
    UE_MAX = 65536;
}

message SignedMsg {
    required SignedEnum value = 1;
}

message UnsignedMsg {
    required UnsignedEnum value = 1;
}
