syntax = "proto2";

import 'nanopb.proto';

message SubMessage
{
    repeated int32 array = 1 [(nanopb).max_count = 8];
}

/* Oneof in a message with other fields */
message AnonymousOneOfMessage
{
    option (nanopb_msgopt).anonymous_oneof = true;
    required int32 prefix = 1;
    oneof values
    {
        int32 first = 5;
        string second = 6 [(nanopb).max_size = 8];
        SubMessage third = 7;
    }
    required int32 suffix = 99;
}

